@@ -17,6 +17,23 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-amrnbparse
+ * @see_also: #GstAmrnbDec, #GstAmrnbEnc
+ *
+ * <refsect2>
+ * <para>
+ * This is an AMR narrowband parser.
+ * </para>
+ * <title>Example launch line</title>
+ * <para>
+ * <programlisting>
+ * gst-launch filesrc location=abc.amr ! amrnbparse ! amrnbdec ! audioresample ! audioconvert ! alsasink
+ * </programlisting>
+ * </para>
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -56,6 +73,8 @@ static gboolean gst_amrnbparse_sink_activate_pull (GstPad * sinkpad,
 static GstStateChangeReturn gst_amrnbparse_state_change (GstElement * element,
     GstStateChange transition);
 
+static void gst_amrnbparse_finalize (GObject * object);
+
 #define _do_init(bla) \
     GST_DEBUG_CATEGORY_INIT (gst_amrnbparse_debug, "amrnbparse", 0, "AMR-NB audio stream parser");
 
@@ -82,8 +101,11 @@ gst_amrnbparse_base_init (gpointer klass)
 static void
 gst_amrnbparse_class_init (GstAmrnbParseClass * klass)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
+  object_class->finalize = gst_amrnbparse_finalize;
+
   element_class->change_state = GST_DEBUG_FUNCPTR (gst_amrnbparse_state_change);
 }
 
@@ -118,6 +140,20 @@ gst_amrnbparse_init (GstAmrnbParse * amrnbparse, GstAmrnbParseClass * klass)
   amrnbparse->ts = 0;
 }
 
+static void
+gst_amrnbparse_finalize (GObject * object)
+{
+  GstAmrnbParse *amrnbparse;
+
+  amrnbparse = GST_AMRNBPARSE (object);
+
+  gst_adapter_clear (amrnbparse->adapter);
+  g_object_unref (amrnbparse->adapter);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
 /*
  * Position querying.
  */
@@ -385,7 +421,7 @@ gst_amrnbparse_loop (GstPad * pad)
   amrnbparse = GST_AMRNBPARSE (GST_PAD_PARENT (pad));
 
   /* init */
-  if (amrnbparse->need_header) {
+  if (G_UNLIKELY (amrnbparse->need_header)) {
     if (!gst_amrnbparse_pull_header (amrnbparse)) {
       GST_ELEMENT_ERROR (amrnbparse, STREAM, WRONG_TYPE, (NULL), (NULL));
       GST_LOG_OBJECT (amrnbparse, "could not read header");
@@ -452,12 +488,22 @@ gst_amrnbparse_loop (GstPad * pad)
 
   ret = gst_pad_push (amrnbparse->srcpad, buffer);
 
-  if (ret != GST_FLOW_OK) {
+  if (G_UNLIKELY (ret != GST_FLOW_OK)) {
     GST_DEBUG_OBJECT (amrnbparse, "Flow: %s", gst_flow_get_name (ret));
-    if (GST_FLOW_IS_FATAL (ret)) {
-      GST_ELEMENT_ERROR (amrnbparse, STREAM, FAILED, (NULL),    /* _("Internal data flow error.")), */
-          ("streaming task paused, reason: %s", gst_flow_get_name (ret)));
-      gst_pad_push_event (pad, gst_event_new_eos ());
+    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+      if (ret == GST_FLOW_UNEXPECTED) {
+        /* we don't do seeking yet, so no segment flag to check here either */
+        if (0) {
+          /* post segment_done message here one day when seeking works */
+        } else {
+          GST_LOG_OBJECT (amrnbparse, "Sending EOS at end of segment");
+          gst_pad_push_event (amrnbparse->srcpad, gst_event_new_eos ());
+        }
+      } else {
+        GST_ELEMENT_ERROR (amrnbparse, STREAM, FAILED, (NULL),
+            ("streaming stopped, reason: %s", gst_flow_get_name (ret)));
+        gst_pad_push_event (amrnbparse->srcpad, gst_event_new_eos ());
+      }
     }
     goto need_pause;
   }
