@@ -38,7 +38,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_id3_tag_debug);
 #define GST_ID3_TAG(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_ID3_TAG, GstID3Tag))
 #define GST_ID3_TAG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_ID3_TAG, GstID3TagClass))
 #define GST_IS_ID3_TAG(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ID3_TAG))
-#define GST_IS_ID3_TAG_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ID3_TAG))
+#define GST_IS_ID3_TAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ID3_TAG))
 #define GST_ID3_TAG_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_ID3_TAG, GstID3TagClass))
 
 typedef struct _GstID3Tag GstID3Tag;
@@ -285,7 +285,7 @@ gst_id3_tag_class_init (gpointer g_class, gpointer class_data)
             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
     gst_element_class_add_pad_template (gstelement_class,
         gst_static_pad_template_get (&id3_tag_src_any_template_factory));
-  } else {
+  } else if (tag_class->type & GST_ID3_TAG_PARSE_MUX) {
     gst_element_class_add_pad_template (gstelement_class,
         gst_static_pad_template_get (&id3_tag_src_id3_template_factory));
   }
@@ -303,7 +303,7 @@ gst_id3_tag_class_init (gpointer g_class, gpointer class_data)
   if (tag_class->type == GST_ID3_TAG_PARSE_MUX) {
     gst_element_class_add_pad_template (gstelement_class,
         gst_static_pad_template_get (&id3_tag_sink_any_template_factory));
-  } else {
+  } else if (tag_class->type & GST_ID3_TAG_PARSE_DEMUX) {
     gst_element_class_add_pad_template (gstelement_class,
         gst_static_pad_template_get (&id3_tag_sink_id3_template_factory));
   }
@@ -1243,7 +1243,7 @@ gst_id3_tag_chain (GstPad * pad, GstBuffer * buffer)
       gst_buffer_unref (buffer);
       if (tag->parse_mode != GST_ID3_TAG_PARSE_ANY) {
         /* seek to beginning */
-        GST_LOG_OBJECT (tag, "seeking back to beginning (offset %d)",
+        GST_LOG_OBJECT (tag, "seeking back to beginning (offset %ld)",
             tag->v2tag_size);
         gst_id3_tag_set_state (tag, GST_ID3_TAG_STATE_SEEKING_TO_NORMAL);
         if (!gst_pad_push_event (tag->sinkpad,
@@ -1276,7 +1276,7 @@ gst_id3_tag_chain (GstPad * pad, GstBuffer * buffer)
       /* Collect a large enough chunk to read the tag */
       if (GST_BUFFER_SIZE (buffer) < tag->v2tag_size) {
         GST_DEBUG_OBJECT (tag,
-            "Not enough data to read ID3v2. Need %d have %d, waiting for more",
+            "Not enough data to read ID3v2. Need %ld have %d, waiting for more",
             tag->v2tag_size, GST_BUFFER_SIZE (buffer));
         tag->buffer = buffer;
         return GST_FLOW_OK;
