@@ -23,76 +23,8 @@
 
 #include <string.h>
 #include "gstmad.h"
+#include <gst/audio/audio.h>
 
-#define GST_TYPE_MAD \
-  (gst_mad_get_type())
-#define GST_MAD(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_MAD,GstMad))
-#define GST_MAD_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_MAD,GstMadClass))
-#define GST_IS_MAD(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_MAD))
-#define GST_IS_MAD_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_MAD))
-
-
-typedef struct _GstMad GstMad;
-typedef struct _GstMadClass GstMadClass;
-
-struct _GstMad
-{
-  GstElement element;
-
-  /* pads */
-  GstPad *sinkpad, *srcpad;
-
-  /* state */
-  struct mad_stream stream;
-  struct mad_frame frame;
-  struct mad_synth synth;
-  guchar *tempbuffer;           /* temporary buffer to serve to mad */
-  glong tempsize;               /* running count of temp buffer size */
-  GstClockTime last_ts;
-  guint64 base_byte_offset;
-  guint64 bytes_consumed;       /* since the base_byte_offset */
-  guint64 total_samples;        /* the number of samples since the sync point */
-
-  gboolean in_error;            /* set when mad's in an error state */
-  gboolean restart;
-  guint64 segment_start;
-  gboolean need_newsegment;
-
-  /* info */
-  struct mad_header header;
-  gboolean new_header;
-  guint framecount;
-  gint vbr_average;             /* average bitrate */
-  guint64 vbr_rate;             /* average * framecount */
-
-  gboolean half;
-  gboolean ignore_crc;
-
-  GstTagList *tags;
-
-  /* negotiated format */
-  gint rate, pending_rate;
-  gint channels, pending_channels;
-  gint times_pending;
-
-  gboolean caps_set;            /* used to keep track of whether to change/update caps */
-  GstIndex *index;
-  gint index_id;
-
-  gboolean check_for_xing;
-  gboolean xing_found;
-
-  gboolean framed;              /* whether there is a demuxer in front of us */
-};
-
-struct _GstMadClass
-{
-  GstElementClass parent_class;
-};
 
 /* elementfactory information */
 static const GstElementDetails gst_mad_details =
@@ -139,7 +71,7 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 
 static void gst_mad_base_init (gpointer g_class);
 static void gst_mad_class_init (GstMadClass * klass);
-static void gst_mad_init (GstMad * mad);
+static void gst_mad_init (GstMad * mad, GstMadClass * klass);
 static void gst_mad_dispose (GObject * object);
 
 static void gst_mad_set_property (GObject * object, guint prop_id,
@@ -166,36 +98,17 @@ static GstStateChangeReturn gst_mad_change_state (GstElement * element,
 static void gst_mad_set_index (GstElement * element, GstIndex * index);
 static GstIndex *gst_mad_get_index (GstElement * element);
 
-
-static GstElementClass *parent_class = NULL;
-
-GType
-gst_mad_get_type (void)
+static void
+_do_init (GType type)
 {
-  static GType mad_type = 0;
-
-  if (!mad_type) {
-    static const GTypeInfo mad_info = {
-      sizeof (GstMadClass),
-      gst_mad_base_init,
-      NULL,
-      (GClassInitFunc) gst_mad_class_init,
-      NULL,
-      NULL,
-      sizeof (GstMad),
-      0,
-      (GInstanceInitFunc) gst_mad_init,
-    };
-
-    mad_type =
-        g_type_register_static (GST_TYPE_ELEMENT, "GstMad", &mad_info, 0);
-  }
   GST_DEBUG_CATEGORY_INIT (mad_debug, "mad", 0, "mad mp3 decoding");
-  return mad_type;
 }
 
+GST_BOILERPLATE_FULL (GstMad, gst_mad, GstElement, GST_TYPE_ELEMENT, _do_init);
+
+/*
 #define GST_TYPE_MAD_LAYER (gst_mad_layer_get_type())
-G_GNUC_UNUSED static GType
+static GType
 gst_mad_layer_get_type (void)
 {
   static GType mad_layer_type = 0;
@@ -212,9 +125,10 @@ gst_mad_layer_get_type (void)
   }
   return mad_layer_type;
 }
+*/
 
 #define GST_TYPE_MAD_MODE (gst_mad_mode_get_type())
-G_GNUC_UNUSED static GType
+static GType
 gst_mad_mode_get_type (void)
 {
   static GType mad_mode_type = 0;
@@ -234,7 +148,7 @@ gst_mad_mode_get_type (void)
 }
 
 #define GST_TYPE_MAD_EMPHASIS (gst_mad_emphasis_get_type())
-G_GNUC_UNUSED static GType
+static GType
 gst_mad_emphasis_get_type (void)
 {
   static GType mad_emphasis_type = 0;
@@ -299,16 +213,21 @@ gst_mad_class_init (GstMadClass * klass)
 #define GST_TAG_MODE     "mode"
 #define GST_TAG_EMPHASIS "emphasis"
 
+  /* FIXME 0.11: strings!? why? */
   gst_tag_register (GST_TAG_LAYER, GST_TAG_FLAG_ENCODED, G_TYPE_UINT,
       "layer", "MPEG audio layer", NULL);
   gst_tag_register (GST_TAG_MODE, GST_TAG_FLAG_ENCODED, G_TYPE_STRING,
       "mode", "MPEG audio channel mode", NULL);
   gst_tag_register (GST_TAG_EMPHASIS, GST_TAG_FLAG_ENCODED, G_TYPE_STRING,
       "emphasis", "MPEG audio emphasis", NULL);
+
+  /* ref these here from a thread-safe context (ie. not the streaming thread) */
+  g_type_class_ref (GST_TYPE_MAD_MODE);
+  g_type_class_ref (GST_TYPE_MAD_EMPHASIS);
 }
 
 static void
-gst_mad_init (GstMad * mad)
+gst_mad_init (GstMad * mad, GstMadClass * klass)
 {
   GstPadTemplate *template;
 
@@ -320,6 +239,7 @@ gst_mad_init (GstMad * mad)
   gst_pad_set_chain_function (mad->sinkpad, GST_DEBUG_FUNCPTR (gst_mad_chain));
   gst_pad_set_event_function (mad->sinkpad,
       GST_DEBUG_FUNCPTR (gst_mad_sink_event));
+
   template = gst_static_pad_template_get (&mad_src_template_factory);
   mad->srcpad = gst_pad_new_from_template (template, "src");
   gst_object_unref (template);
@@ -331,6 +251,7 @@ gst_mad_init (GstMad * mad)
   gst_pad_set_query_type_function (mad->srcpad,
       GST_DEBUG_FUNCPTR (gst_mad_get_query_types));
   gst_pad_use_fixed_caps (mad->srcpad);
+
   mad->tempbuffer = g_malloc (MAD_BUFFER_MDLEN * 3);
   mad->tempsize = 0;
   mad->base_byte_offset = 0;
@@ -342,6 +263,7 @@ gst_mad_init (GstMad * mad)
   mad->vbr_rate = 0;
   mad->restart = TRUE;
   mad->segment_start = 0;
+  gst_segment_init (&mad->segment, GST_FORMAT_TIME);
   mad->header.mode = -1;
   mad->header.emphasis = -1;
   mad->tags = NULL;
@@ -359,10 +281,14 @@ gst_mad_dispose (GObject * object)
 
   gst_mad_set_index (GST_ELEMENT (object), NULL);
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
-
   g_free (mad->tempbuffer);
   mad->tempbuffer = NULL;
+
+  g_list_foreach (mad->pending_events, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (mad->pending_events);
+  mad->pending_events = NULL;
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -458,7 +384,7 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
 
   mad = GST_MAD (GST_PAD_PARENT (pad));
 
-  bytes_per_sample = MAD_NCHANNELS (&mad->frame.header) * 4;
+  bytes_per_sample = mad->channels * 4;
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
@@ -470,7 +396,7 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
           break;
         case GST_FORMAT_TIME:
         {
-          gint byterate = bytes_per_sample * mad->frame.header.samplerate;
+          gint byterate = bytes_per_sample * mad->rate;
 
           if (byterate == 0)
             return FALSE;
@@ -488,10 +414,10 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
           *dest_value = src_value * bytes_per_sample;
           break;
         case GST_FORMAT_TIME:
-          if (mad->frame.header.samplerate == 0)
+          if (mad->rate == 0)
             return FALSE;
           *dest_value = gst_util_uint64_scale_int (src_value, GST_SECOND,
-              mad->frame.header.samplerate);
+              mad->rate);
           break;
         default:
           res = FALSE;
@@ -504,7 +430,7 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
           /* fallthrough */
         case GST_FORMAT_DEFAULT:
           *dest_value = gst_util_uint64_scale_int (src_value,
-              scale * mad->frame.header.samplerate, GST_SECOND);
+              scale * mad->rate, GST_SECOND);
           break;
         default:
           res = FALSE;
@@ -690,6 +616,13 @@ index_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   gst_event_parse_seek (event, &rate, &format, &flags,
       &cur_type, &cur, &stop_type, &stop);
 
+  if (format == GST_FORMAT_TIME) {
+    gst_segment_set_seek (&mad->segment, rate, format, flags, cur_type,
+        cur, stop_type, stop, NULL);
+  } else {
+    gst_segment_init (&mad->segment, GST_FORMAT_UNDEFINED);
+  }
+
   entry = gst_index_get_assoc_entry (mad->index, mad->index_id,
       GST_INDEX_LOOKUP_BEFORE, 0, format, cur);
 
@@ -764,6 +697,9 @@ normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
     time_stop = stop;
   }
 
+  gst_segment_set_seek (&mad->segment, rate, GST_FORMAT_TIME, flags, cur_type,
+      time_cur, stop_type, time_stop, NULL);
+
   GST_DEBUG ("seek to time %" GST_TIME_FORMAT "-%" GST_TIME_FORMAT,
       GST_TIME_ARGS (time_cur), GST_TIME_ARGS (time_stop));
 
@@ -973,10 +909,10 @@ G_STMT_START{                                                   \
     GEnumValue *emphasis;
 
     mode =
-        g_enum_get_value (g_type_class_ref (GST_TYPE_MAD_MODE),
+        g_enum_get_value (g_type_class_peek (GST_TYPE_MAD_MODE),
         mad->header.mode);
     emphasis =
-        g_enum_get_value (g_type_class_ref (GST_TYPE_MAD_EMPHASIS),
+        g_enum_get_value (g_type_class_peek (GST_TYPE_MAD_EMPHASIS),
         mad->header.emphasis);
     list = gst_tag_list_new ();
     gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
@@ -1013,9 +949,12 @@ gst_mad_sink_event (GstPad * pad, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:{
       GstFormat format;
+      gboolean update;
+      gdouble rate, applied_rate;
+      gint64 start, stop, pos;
 
-      gst_event_parse_new_segment (event, NULL, NULL, &format, NULL, NULL,
-          NULL);
+      gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
+          &format, &start, &stop, &pos);
 
       if (format == GST_FORMAT_TIME) {
         /* FIXME: is this really correct? */
@@ -1024,6 +963,8 @@ gst_mad_sink_event (GstPad * pad, GstEvent * event)
         /* we don't need to restart when we get here */
         mad->restart = FALSE;
         mad->framed = TRUE;
+        gst_segment_set_newsegment_full (&mad->segment, update, rate,
+            applied_rate, GST_FORMAT_TIME, start, stop, pos);
       } else {
         GST_DEBUG ("dropping newsegment event in format %s",
             gst_format_get_name (format));
@@ -1047,11 +988,18 @@ gst_mad_sink_event (GstPad * pad, GstEvent * event)
       mad->tempsize = 0;
       mad_frame_mute (&mad->frame);
       mad_synth_mute (&mad->synth);
+    case GST_EVENT_FLUSH_START:
       result = gst_pad_event_default (pad, event);
 
       break;
     default:
-      result = gst_pad_event_default (pad, event);
+      if (mad->restart) {
+        /* Cache all other events if we still have to send a NEWSEGMENT */
+        mad->pending_events = g_list_append (mad->pending_events, event);
+        result = TRUE;
+      } else {
+        result = gst_pad_event_default (pad, event);
+      }
       break;
   }
   return result;
@@ -1171,14 +1119,25 @@ mpg123_parse_xing_header (struct mad_header *header,
       ptr += 4;
     }
     if (xflags & XING_TOC_FLAG) {
+      guchar old = 0;
+
       lprintf ("toc found\n");
       if (ptr >= (buf + bufsize - XING_TOC_LENGTH))
         return 0;
+      if (*ptr != 0) {
+        lprintf ("skipping broken Xing TOC\n");
+        goto skip_toc;
+      }
       for (i = 0; i < XING_TOC_LENGTH; i++) {
         xtoc[i] = *(ptr + i);
+        if (old > xtoc[i]) {
+          lprintf ("skipping broken Xing TOC\n");
+          goto skip_toc;
+        }
         lprintf ("%d ", xtoc[i]);
       }
       lprintf ("\n");
+    skip_toc:
       ptr += XING_TOC_LENGTH;
     }
 
@@ -1461,7 +1420,12 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
               } else {
                 mad->tags = gst_tag_list_copy (list);
               }
-              gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
+              if (mad->need_newsegment)
+                mad->pending_events =
+                    g_list_append (mad->pending_events,
+                    gst_event_new_tag (list));
+              else
+                gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
             }
           }
         }
@@ -1501,7 +1465,12 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
               GST_TAG_BITRATE, bitrate, NULL);
           gst_element_post_message (GST_ELEMENT (mad),
               gst_message_new_tag (GST_OBJECT (mad), gst_tag_list_copy (list)));
-          gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
+
+          if (mad->need_newsegment)
+            mad->pending_events =
+                g_list_append (mad->pending_events, gst_event_new_tag (list));
+          else
+            gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
 
           goto next_no_samples;
         }
@@ -1513,9 +1482,8 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
       nsamples = MAD_NSBSAMPLES (&mad->frame.header) *
           (mad->stream.options & MAD_OPTION_HALFSAMPLERATE ? 16 : 32);
 
-      if (mad->frame.header.samplerate == 0) {
-        g_warning
-            ("mad->frame.header.samplerate is 0; timestamps cannot be calculated");
+      if (mad->rate == 0) {
+        g_warning ("mad->rate is 0; timestamps cannot be calculated");
       } else {
         /* if we have a pending timestamp, we can use it now to calculate the sample offset */
         if (GST_CLOCK_TIME_IS_VALID (mad->last_ts)) {
@@ -1585,12 +1553,25 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
           GST_DEBUG ("Sending NEWSEGMENT event, start=%" GST_TIME_FORMAT,
               GST_TIME_ARGS (start));
 
+          gst_segment_set_newsegment (&mad->segment, FALSE, 1.0,
+              GST_FORMAT_TIME, start, GST_CLOCK_TIME_NONE, start);
+
           gst_pad_push_event (mad->srcpad,
               gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME,
                   start, GST_CLOCK_TIME_NONE, start));
           mad->need_newsegment = FALSE;
         }
 
+        if (mad->pending_events) {
+          GList *l;
+
+          for (l = mad->pending_events; l != NULL; l = l->next) {
+            gst_pad_push_event (mad->srcpad, GST_EVENT (l->data));
+          }
+          g_list_free (mad->pending_events);
+          mad->pending_events = NULL;
+        }
+
         /* will attach the caps to the buffer */
         result =
             gst_pad_alloc_buffer_and_set_caps (mad->srcpad, 0,
@@ -1632,10 +1613,20 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
           }
         }
 
-        result = gst_pad_push (mad->srcpad, outbuffer);
-        if (result != GST_FLOW_OK) {
-          /* Head for the exit, dropping samples as we go */
-          goto_exit = TRUE;
+        if ((outbuffer = gst_audio_buffer_clip (outbuffer, &mad->segment,
+                    mad->rate, 4 * mad->channels))) {
+          GST_LOG_OBJECT (mad,
+              "pushing buffer, off=%" G_GUINT64_FORMAT ", ts=%" GST_TIME_FORMAT,
+              GST_BUFFER_OFFSET (outbuffer),
+              GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuffer)));
+          mad->segment.last_stop = GST_BUFFER_TIMESTAMP (outbuffer);
+          result = gst_pad_push (mad->srcpad, outbuffer);
+          if (result != GST_FLOW_OK) {
+            /* Head for the exit, dropping samples as we go */
+            goto_exit = TRUE;
+          }
+        } else {
+          GST_LOG_OBJECT (mad, "Dropping buffer");
         }
       }
 
@@ -1708,7 +1699,7 @@ gst_mad_change_state (GstElement * element, GstStateChange transition)
       mad->caps_set = FALSE;
       mad->times_pending = mad->pending_rate = mad->pending_channels = 0;
       mad->vbr_average = 0;
-      mad->segment_start = 0;
+      gst_segment_init (&mad->segment, GST_FORMAT_TIME);
       mad->new_header = TRUE;
       mad->framed = FALSE;
       mad->framecount = 0;
