@@ -29,8 +29,35 @@
 GST_DEBUG_CATEGORY_STATIC (mp3parse_debug);
 #define GST_CAT_DEFAULT mp3parse_debug
 
+#define MP3_CHANNEL_MODE_UNKNOWN -1
+#define MP3_CHANNEL_MODE_STEREO 0
+#define MP3_CHANNEL_MODE_JOINT_STEREO 1
+#define MP3_CHANNEL_MODE_DUAL_CHANNEL 2
+#define MP3_CHANNEL_MODE_MONO 3
+
+#define CRC_UNKNOWN -1
+#define CRC_PROTECTED 0
+#define CRC_NOT_PROTECTED 1
+
+#define XING_FRAMES_FLAG     0x0001
+#define XING_BYTES_FLAG      0x0002
+#define XING_TOC_FLAG        0x0004
+#define XING_VBR_SCALE_FLAG  0x0008
+
 #define GST_READ_UINT24_BE(p) (p[2] | (p[1] << 8) | (p[0] << 16))
 
+static inline MPEGAudioSeekEntry *
+mpeg_audio_seek_entry_new ()
+{
+  return g_slice_new (MPEGAudioSeekEntry);
+}
+
+static inline void
+mpeg_audio_seek_entry_free (MPEGAudioSeekEntry * entry)
+{
+  g_slice_free (MPEGAudioSeekEntry, entry);
+}
+
 /* elementfactory information */
 static GstElementDetails mp3parse_details = {
   "MPEG1 Audio Parser",
@@ -100,10 +127,31 @@ mp3parse_total_bytes (GstMPEGAudioParse * mp3parse, gint64 * total);
 static gboolean
 mp3parse_total_time (GstMPEGAudioParse * mp3parse, GstClockTime * total);
 
-/*static guint gst_mp3parse_signals[LAST_SIGNAL] = { 0 }; */
+/* static guint gst_mp3parse_signals[LAST_SIGNAL] = { 0 }; */
 
 GST_BOILERPLATE (GstMPEGAudioParse, gst_mp3parse, GstElement, GST_TYPE_ELEMENT);
 
+#define GST_TYPE_MP3_CHANNEL_MODE (gst_mp3_channel_mode_get_type())
+G_GNUC_UNUSED static GType
+gst_mp3_channel_mode_get_type (void)
+{
+  static GType mp3_channel_mode_type = 0;
+  static GEnumValue mp3_channel_mode[] = {
+    {MP3_CHANNEL_MODE_UNKNOWN, "Unknown", "unknown"},
+    {MP3_CHANNEL_MODE_MONO, "Mono", "mono"},
+    {MP3_CHANNEL_MODE_DUAL_CHANNEL, "Dual Channel", "dual-channel"},
+    {MP3_CHANNEL_MODE_JOINT_STEREO, "Joint Stereo", "joint-stereo"},
+    {MP3_CHANNEL_MODE_STEREO, "Stereo", "stereo"},
+    {0, NULL, NULL},
+  };
+
+  if (!mp3_channel_mode_type) {
+    mp3_channel_mode_type =
+        g_enum_register_static ("GstMp3ChannelMode", mp3_channel_mode);
+  }
+  return mp3_channel_mode_type;
+}
+
 static const guint mp3types_bitrates[2][3][16] = {
   {
         {0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448,},
@@ -125,11 +173,13 @@ static const guint mp3types_freqs[3][3] = { {44100, 48000, 32000},
 static inline guint
 mp3_type_frame_length_from_header (GstMPEGAudioParse * mp3parse, guint32 header,
     guint * put_version, guint * put_layer, guint * put_channels,
-    guint * put_bitrate, guint * put_samplerate)
+    guint * put_bitrate, guint * put_samplerate, guint * put_mode,
+    guint * put_crc)
 {
   guint length;
-  gulong mode, samplerate, bitrate, layer, channels, padding;
+  gulong mode, samplerate, bitrate, layer, channels, padding, crc;
   gint lsf, mpg25;
+  GEnumValue *mode_enum;
 
   if (header & (1 << 20)) {
     lsf = (header & (1 << 19)) ? 0 : 1;
@@ -141,6 +191,8 @@ mp3_type_frame_length_from_header (GstMPEGAudioParse * mp3parse, guint32 header,
 
   layer = 4 - ((header >> 17) & 0x3);
 
+  crc = (header >> 16) & 0x1;
+
   bitrate = (header >> 12) & 0xF;
   bitrate = mp3types_bitrates[lsf][layer - 1][bitrate] * 1000;
   if (bitrate == 0)
@@ -167,10 +219,14 @@ mp3_type_frame_length_from_header (GstMPEGAudioParse * mp3parse, guint32 header,
       break;
   }
 
+  mode_enum =
+      g_enum_get_value (g_type_class_ref (GST_TYPE_MP3_CHANNEL_MODE), mode);
+
   GST_DEBUG_OBJECT (mp3parse, "Calculated mp3 frame length of %u bytes",
       length);
   GST_DEBUG_OBJECT (mp3parse, "samplerate = %lu, bitrate = %lu, layer = %lu, "
-      "channels = %lu", samplerate, bitrate, layer, channels);
+      "channels = %lu, mode = %s", samplerate, bitrate, layer, channels,
+      mode_enum->value_nick);
 
   if (put_version)
     *put_version = lsf ? 2 : 1;
@@ -182,6 +238,10 @@ mp3_type_frame_length_from_header (GstMPEGAudioParse * mp3parse, guint32 header,
     *put_bitrate = bitrate;
   if (put_samplerate)
     *put_samplerate = samplerate;
+  if (put_mode)
+    *put_mode = mode;
+  if (put_crc)
+    *put_crc = crc;
 
   return length;
 }
@@ -242,6 +302,15 @@ gst_mp3parse_class_init (GstMPEGAudioParseClass * klass)
           G_MININT, G_MAXINT, 0, G_PARAM_READABLE));
 
   gstelement_class->change_state = gst_mp3parse_change_state;
+
+/* register tags */
+#define GST_TAG_CRC    "has-crc"
+#define GST_TAG_MODE     "channel-mode"
+
+  gst_tag_register (GST_TAG_CRC, GST_TAG_FLAG_META, G_TYPE_BOOLEAN,
+      "has crc", "Using CRC", NULL);
+  gst_tag_register (GST_TAG_MODE, GST_TAG_FLAG_ENCODED, G_TYPE_STRING,
+      "channel mode", "MPEG audio channel mode", NULL);
 }
 
 static void
@@ -268,6 +337,9 @@ gst_mp3parse_reset (GstMPEGAudioParse * mp3parse)
   mp3parse->frame_count = 0;
   mp3parse->sent_codec_tag = FALSE;
 
+  mp3parse->last_posted_crc = CRC_UNKNOWN;
+  mp3parse->last_posted_channel_mode = MP3_CHANNEL_MODE_UNKNOWN;
+
   mp3parse->xing_flags = 0;
   mp3parse->xing_bitrate = 0;
   mp3parse->xing_frames = 0;
@@ -286,7 +358,8 @@ gst_mp3parse_reset (GstMPEGAudioParse * mp3parse)
   mp3parse->vbri_seek_table = NULL;
 
   if (mp3parse->seek_table) {
-    g_list_foreach (mp3parse->seek_table, (GFunc) g_free, NULL);
+    g_list_foreach (mp3parse->seek_table, (GFunc) mpeg_audio_seek_entry_free,
+        NULL);
     g_list_free (mp3parse->seek_table);
     mp3parse->seek_table = NULL;
   }
@@ -346,13 +419,18 @@ gst_mp3parse_dispose (GObject * object)
   g_mutex_free (mp3parse->pending_accurate_seeks_lock);
   mp3parse->pending_accurate_seeks_lock = NULL;
 
+  g_list_foreach (mp3parse->pending_events, (GFunc) gst_mini_object_unref,
+      NULL);
+  g_list_free (mp3parse->pending_events);
+  mp3parse->pending_events = NULL;
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static gboolean
 gst_mp3parse_sink_event (GstPad * pad, GstEvent * event)
 {
-  gboolean res;
+  gboolean res = TRUE;
   GstMPEGAudioParse *mp3parse;
   GstEvent **eventp;
 
@@ -402,7 +480,7 @@ gst_mp3parse_sink_event (GstPad * pad, GstEvent * event)
 
           GST_DEBUG_OBJECT (mp3parse,
               "Pushing accurate newseg rate %g, applied rate %g, "
-              "format %d, start %lld, stop %lld, pos %lld\n", rate,
+              "format %d, start %lld, stop %lld, pos %lld", rate,
               applied_rate, format, start, stop, pos);
 
           g_free (seek);
@@ -458,7 +536,7 @@ gst_mp3parse_sink_event (GstPad * pad, GstEvent * event)
       gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
           &format, &start, &stop, &pos);
       GST_DEBUG_OBJECT (mp3parse, "Pushing newseg rate %g, applied rate %g, "
-          "format %d, start %lld, stop %lld, pos %lld\n",
+          "format %d, start %lld, stop %lld, pos %lld",
           rate, applied_rate, format, start, stop, pos);
 
       gst_segment_set_newsegment_full (&mp3parse->segment, update, rate,
@@ -480,7 +558,16 @@ gst_mp3parse_sink_event (GstPad * pad, GstEvent * event)
       res = gst_pad_push_event (mp3parse->srcpad, event);
       break;
     default:
-      res = gst_pad_push_event (mp3parse->srcpad, event);
+      if (mp3parse->pending_segment &&
+          (GST_EVENT_TYPE (event) != GST_EVENT_EOS) &&
+          (GST_EVENT_TYPE (event) != GST_EVENT_FLUSH_START)) {
+        /* Cache all events except EOS and the ones above if we have
+         * a pending segment */
+        mp3parse->pending_events =
+            g_list_append (mp3parse->pending_events, event);
+      } else {
+        res = gst_pad_push_event (mp3parse->srcpad, event);
+      }
       break;
   }
 
@@ -503,12 +590,14 @@ mp3parse_seek_table_last_entry (GstMPEGAudioParse * mp3parse)
 
 /* Prepare a buffer of the indicated size, timestamp it and output */
 static GstFlowReturn
-gst_mp3parse_emit_frame (GstMPEGAudioParse * mp3parse, guint size)
+gst_mp3parse_emit_frame (GstMPEGAudioParse * mp3parse, guint size,
+    guint mode, guint crc)
 {
   GstBuffer *outbuf;
   guint bitrate;
   GstFlowReturn ret = GST_FLOW_OK;
   GstClockTime push_start;
+  GstTagList *taglist;
 
   outbuf = gst_adapter_take_buffer (mp3parse->adapter, size);
 
@@ -553,7 +642,8 @@ gst_mp3parse_emit_frame (GstMPEGAudioParse * mp3parse, guint size)
 
     /* No timestamp yet, convert our offset to a timestamp if we can, or
      * start at 0 */
-    if (mp3parse_bytepos_to_time (mp3parse, mp3parse->cur_offset, &ts))
+    if (mp3parse_bytepos_to_time (mp3parse, mp3parse->cur_offset, &ts) &&
+        GST_CLOCK_TIME_IS_VALID (ts))
       GST_BUFFER_TIMESTAMP (outbuf) = ts;
     else {
       GST_BUFFER_TIMESTAMP (outbuf) = 0;
@@ -568,7 +658,7 @@ gst_mp3parse_emit_frame (GstMPEGAudioParse * mp3parse, guint size)
       (!mp3parse->seek_table ||
           (mp3parse_seek_table_last_entry (mp3parse))->byte <
           GST_BUFFER_OFFSET (outbuf))) {
-    MPEGAudioSeekEntry *entry = g_new0 (MPEGAudioSeekEntry, 1);
+    MPEGAudioSeekEntry *entry = mpeg_audio_seek_entry_new ();
 
     entry->byte = mp3parse->cur_offset;
     entry->timestamp = GST_BUFFER_TIMESTAMP (outbuf);
@@ -599,12 +689,59 @@ gst_mp3parse_emit_frame (GstMPEGAudioParse * mp3parse, guint size)
   else
     bitrate = mp3parse->avg_bitrate;
 
+  /* we will create a taglist (if any of the parameters has changed)
+   * to add the tags that changed */
+  taglist = NULL;
   if ((mp3parse->last_posted_bitrate / 10000) != (bitrate / 10000)) {
-    GstTagList *taglist = gst_tag_list_new ();
-
+    taglist = gst_tag_list_new ();
     mp3parse->last_posted_bitrate = bitrate;
     gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE, GST_TAG_BITRATE,
         mp3parse->last_posted_bitrate, NULL);
+
+    /* Post a new duration message if the average bitrate changes that much
+     * so applications can update their cached values
+     */
+    if ((mp3parse->xing_flags & XING_TOC_FLAG) == 0
+        && mp3parse->vbri_total_time == 0) {
+      gst_element_post_message (GST_ELEMENT (mp3parse),
+          gst_message_new_duration (GST_OBJECT (mp3parse), GST_FORMAT_TIME,
+              -1));
+    }
+  }
+
+  if (mp3parse->last_posted_crc != crc) {
+    gboolean using_crc;
+
+    if (!taglist) {
+      taglist = gst_tag_list_new ();
+    }
+    mp3parse->last_posted_crc = crc;
+    if (mp3parse->last_posted_crc == CRC_PROTECTED) {
+      using_crc = TRUE;
+    } else {
+      using_crc = FALSE;
+    }
+    gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE, GST_TAG_CRC,
+        using_crc, NULL);
+  }
+
+  if (mp3parse->last_posted_channel_mode != mode) {
+    GEnumValue *mode_enum;
+
+    if (!taglist) {
+      taglist = gst_tag_list_new ();
+    }
+    mp3parse->last_posted_channel_mode = mode;
+
+    mode_enum = g_enum_get_value (g_type_class_ref (GST_TYPE_MP3_CHANNEL_MODE),
+        mp3parse->last_posted_channel_mode);
+
+    gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE, GST_TAG_MODE,
+        mode_enum->value_nick, NULL);
+  }
+
+  /* if the taglist exists, we need to send it */
+  if (taglist) {
     gst_element_found_tags_for_pad (GST_ELEMENT (mp3parse),
         mp3parse->srcpad, taglist);
   }
@@ -614,11 +751,11 @@ gst_mp3parse_emit_frame (GstMPEGAudioParse * mp3parse, guint size)
    * 9 (29) frames are the theoretical maximum of frames that contain
    * data for the current frame (bit reservoir).
    */
-
   if (mp3parse->segment.start == 0) {
     push_start = 0;
   } else if (GST_CLOCK_TIME_IS_VALID (mp3parse->max_bitreservoir)) {
-    if (mp3parse->segment.start > mp3parse->max_bitreservoir)
+    if (GST_CLOCK_TIME_IS_VALID (mp3parse->segment.start) &&
+        mp3parse->segment.start > mp3parse->max_bitreservoir)
       push_start = mp3parse->segment.start - mp3parse->max_bitreservoir;
     else
       push_start = 0;
@@ -630,18 +767,34 @@ gst_mp3parse_emit_frame (GstMPEGAudioParse * mp3parse, guint size)
               GST_BUFFER_TIMESTAMP_IS_VALID (outbuf) &&
               GST_BUFFER_DURATION_IS_VALID (outbuf) &&
               GST_BUFFER_TIMESTAMP (outbuf) + GST_BUFFER_DURATION (outbuf)
-              < push_start)
-          || (GST_CLOCK_TIME_IS_VALID (mp3parse->segment.stop)
-              && GST_BUFFER_TIMESTAMP (outbuf) >= mp3parse->segment.stop))) {
+              < push_start))) {
     GST_DEBUG_OBJECT (mp3parse,
-        "Buffer outside of configured segment range %" GST_TIME_FORMAT
+        "Buffer before configured segment range %" GST_TIME_FORMAT
         " to %" GST_TIME_FORMAT ", dropping, timestamp %"
-        GST_TIME_FORMAT ", offset 0x%08" G_GINT64_MODIFIER "x",
-        GST_TIME_ARGS (push_start), GST_TIME_ARGS (mp3parse->segment.stop),
+        GST_TIME_FORMAT " duration %" GST_TIME_FORMAT
+        ", offset 0x%08" G_GINT64_MODIFIER "x", GST_TIME_ARGS (push_start),
+        GST_TIME_ARGS (mp3parse->segment.stop),
         GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
+        GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)),
         GST_BUFFER_OFFSET (outbuf));
+
     gst_buffer_unref (outbuf);
     ret = GST_FLOW_OK;
+  } else if (G_UNLIKELY (GST_BUFFER_TIMESTAMP_IS_VALID (outbuf) &&
+          GST_CLOCK_TIME_IS_VALID (mp3parse->segment.stop) &&
+          GST_BUFFER_TIMESTAMP (outbuf) >= mp3parse->segment.stop)) {
+    GST_DEBUG_OBJECT (mp3parse,
+        "Buffer after configured segment range %" GST_TIME_FORMAT
+        " to %" GST_TIME_FORMAT ", returning GST_FLOW_UNEXPECTED, timestamp %"
+        GST_TIME_FORMAT " duration %" GST_TIME_FORMAT ", offset 0x%08"
+        G_GINT64_MODIFIER "x", GST_TIME_ARGS (push_start),
+        GST_TIME_ARGS (mp3parse->segment.stop),
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
+        GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)),
+        GST_BUFFER_OFFSET (outbuf));
+
+    gst_buffer_unref (outbuf);
+    ret = GST_FLOW_UNEXPECTED;
   } else {
     GST_DEBUG_OBJECT (mp3parse,
         "pushing buffer of %d bytes, timestamp %" GST_TIME_FORMAT
@@ -654,17 +807,21 @@ gst_mp3parse_emit_frame (GstMPEGAudioParse * mp3parse, guint size)
       gst_pad_push_event (mp3parse->srcpad, mp3parse->pending_segment);
       mp3parse->pending_segment = NULL;
     }
+    if (mp3parse->pending_events) {
+      GList *l;
+
+      for (l = mp3parse->pending_events; l != NULL; l = l->next) {
+        gst_pad_push_event (mp3parse->srcpad, GST_EVENT (l->data));
+      }
+      g_list_free (mp3parse->pending_events);
+      mp3parse->pending_events = NULL;
+    }
     ret = gst_pad_push (mp3parse->srcpad, outbuf);
   }
 
   return ret;
 }
 
-#define XING_FRAMES_FLAG     0x0001
-#define XING_BYTES_FLAG      0x0002
-#define XING_TOC_FLAG        0x0004
-#define XING_VBR_SCALE_FLAG  0x0008
-
 static void
 gst_mp3parse_handle_first_frame (GstMPEGAudioParse * mp3parse)
 {
@@ -1012,8 +1169,16 @@ gst_mp3parse_chain (GstPad * pad, GstBuffer * buf)
     mp3parse->pending_ts = timestamp;
     mp3parse->pending_offset = mp3parse->tracked_offset + avail;
 
+    /* If we have no data pending and the next timestamp is
+     * invalid we can use the upstream timestamp for the next frame.
+     *
+     * This will give us a timestamp if we're resyncing and upstream
+     * gave us -1 as offset. */
+    if (avail == 0 && !GST_CLOCK_TIME_IS_VALID (mp3parse->next_ts))
+      mp3parse->next_ts = timestamp;
+
     GST_LOG_OBJECT (mp3parse, "Have pending ts %" GST_TIME_FORMAT
-        " to apply in %lld bytes (@ off %lld)\n",
+        " to apply in %lld bytes (@ off %lld)",
         GST_TIME_ARGS (mp3parse->pending_ts), avail, mp3parse->pending_offset);
   }
 
@@ -1045,10 +1210,11 @@ gst_mp3parse_chain (GstPad * pad, GstBuffer * buf)
     header = GST_READ_UINT32_BE (data);
     /* if it's a valid header, go ahead and send off the frame */
     if (head_check (mp3parse, header)) {
-      guint bitrate = 0, layer = 0, rate = 0, channels = 0, version = 0;
+      guint bitrate = 0, layer = 0, rate = 0, channels = 0, version = 0, mode =
+          0, crc = 0;
 
       if (!(bpf = mp3_type_frame_length_from_header (mp3parse, header,
-                  &version, &layer, &channels, &bitrate, &rate)))
+                  &version, &layer, &channels, &bitrate, &rate, &mode, &crc)))
         goto header_error;
 
       /*************************************************************************
@@ -1154,7 +1320,7 @@ gst_mp3parse_chain (GstPad * pad, GstBuffer * buf)
 
       if (!mp3parse->skip) {
         mp3parse->resyncing = FALSE;
-        flow = gst_mp3parse_emit_frame (mp3parse, bpf);
+        flow = gst_mp3parse_emit_frame (mp3parse, bpf, mode, crc);
       } else {
         GST_DEBUG_OBJECT (mp3parse, "skipping buffer of %d bytes", bpf);
         gst_adapter_flush (mp3parse->adapter, bpf);
